<?php

namespace App\Http\Controllers;

use App\Models\Product;
use App\Models\Transaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Log; // Import Log facade
use App\Models\ProductEntry;
use App\Models\LowStockAlert;
use Illuminate\Support\Facades\DB;
use Exception;
use Termwind\Components\Dd;
use App\Models\Supplier;


class TransactionController extends Controller
{
    public function index(Request $request)
    {

        $transactions = Transaction::with(['product', 'productEntries'])->where('type', 'exit')->get();
        $entries = Transaction::with(['product', 'productEntries'])->where('type', 'entry')->get();
        $products = Product::all();
        $suppliers = Supplier::all();
        foreach ($entries as $transaction) {
            $productEntry = $transaction->productEntries->first();
            if ($productEntry) {
                // Log::info('Product Entry Price:', ['price' => $productEntry->price]); // Debugging log
            } else {
                // Log::info('No Product Entry found for transaction:', ['transaction_id' => $transaction->id]); // Debugging log
            }
        }
        return view('transactions.index', compact('transactions', 'products', 'suppliers', 'entries'));
    }


    public function store(Request $request)
    {
        $user = Auth::user();

        Log::info('Entering store method', ['userID' => Auth::id()]);
        Log::info('try', ['userID' => Auth::id()]);
        Log::info('Request data', $request->all());

        $validatedData = $request->validate([
            'quantity' => 'required|integer|min:1',
            'type' => 'required|in:entry,exit',
            'remarks' => 'nullable|string',
            'product_name' => 'required|string|exists:products,name',
            'expiry_date' => 'required_if:type,entry|date|nullable',
            'price' => 'required_if:type,entry|nullable|numeric|min:0',
            'supplier_id' => 'required_if:type,entry|exists:suppliers,id' // Add validation for supplier
        ]);
        Log::info('salit data validation', ['userID' => Auth::id()]);

        $product = Product::where('name', $request['product_name'])->first();

        if (!$product) {
            Log::error('Product not found', ['product_name' => $request['product_name']]);
            return back()->withErrors(['product_name' => 'The selected product does not exist.'])->withInput();
        }

        DB::beginTransaction();

        try {
            Log::info('dkhlt try', ['userID' => Auth::id()]);

            if ($validatedData['type'] == 'entry') {
                $productEntry = ProductEntry::create([
                    'product_id' => $product->id,
                    'quantity' => $validatedData['quantity'],
                    'expiry_date' => $validatedData['expiry_date'],
                    'price' => $validatedData['price'], // Set the price
                    'supplier_id' => $validatedData['supplier_id'] // Set the supplier_id
                ]);

                // Create the transaction
                $transaction = Transaction::create(array_merge($validatedData, [
                    'product_id' => $product->id,
                    'user_id' => $user->id,
                    'user_name' => $user->name,
                ]));

                // Attach product entry to transaction
                $transaction->productEntries()->attach($productEntry->id, ['quantity' => $validatedData['quantity']]);

                Log::info('Entry transaction processed', ['product_entry_id' => $productEntry->id, 'quantity' => $validatedData['quantity']]);
            } else {
                Log::info('else');

                $remainingQuantity = $validatedData['quantity'];
                $totalAvailableQuantity = ProductEntry::where('product_id', $product->id)
                    ->where('quantity', '>', 0)
                    ->sum('quantity');
                Log::info('Total available quantity', ['totalAvailableQuantity' => $totalAvailableQuantity]);
                if ($remainingQuantity > $totalAvailableQuantity) {
                    Log::error('Insufficient stock for exit transaction', ['product_id' => $product->id, 'remainingQuantity' => $remainingQuantity]);
                    return back()->with([
                        'message' => 'The quantity for the exit transaction exceeds the available stock.',
                        'alert-type' => 'error'
                    ]);
                }

                $productEntries = ProductEntry::where('product_id', $product->id)
                    ->where('quantity', '>', 0)
                    ->orderBy('expiry_date', 'asc')
                    ->get();
                $transaction = Transaction::create(array_merge($validatedData, [
                    'product_id' => $product->id,
                    'user_id' => $user->id,
                    'user_name' => $user->name,
                ]));

                foreach ($productEntries as $entry) {
                    if ($remainingQuantity <= 0) break;

                    $quantityTaken = min($remainingQuantity, $entry->quantity);
                    $transaction->productEntries()->attach($entry->id, ['quantity' => $quantityTaken]);

                    $entry->decrement('quantity', $quantityTaken);
                    $remainingQuantity -= $quantityTaken;

                    Log::info('Exit transaction processed', ['product_entry_id' => $entry->id, 'quantity' => $quantityTaken]);
                }
            }

            DB::commit();

            // Recalculate the current stock
            $entries = $product->transactions()->where('type', 'entry')->sum('quantity');
            $exits = $product->transactions()->where('type', 'exit')->sum('quantity');
            $currentStock = $entries - $exits;

            // Check if product is below safety stock level and send email
            if ($currentStock < $product->safety_stock_level && !$product->low_stock_email_sent) {
                $mailController = new MailController();
                $mailController->send_low_stock_mail($product);
                $product->low_stock_email_sent = true;
                $product->save();
                LowStockAlert::create(['product_id' => $product->id]);
            }

            // Handle above safety stock level
            if ($currentStock > $product->safety_stock_level) {
                if ($product->low_stock_email_sent) {
                    $product->low_stock_email_sent = false;
                    $product->save();
                }

                $unresolvedAlert = LowStockAlert::where('product_id', $product->id)
                    ->where('resolved', false)
                    ->first();
                if ($unresolvedAlert) {
                    $unresolvedAlert->resolved = true;
                    $unresolvedAlert->save();
                }
            }

            Log::info('Transaction completed successfully', ['product_id' => $product->id, 'transaction_type' => $validatedData['type']]);
            return redirect()->route('transactions.index')->with([
                'message' => 'Transaction successfully processed.',
                'alert-type' => 'success'
            ]);
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Transaction store failed', ['error' => $e->getMessage()]);
            return back()->with([
                'message' => 'Failed to delete transaction: ' . $e->getMessage(),
                'alert-type' => 'error'
            ]);
        }
    }



    public function edit(Transaction $transaction)
    {
        $products = Product::all();
        $expiryDate = null;
        if ($transaction->type == 'entry') {
            Log::info("rah kanqlb entry");

            // Fetch the corresponding ProductEntry for this transaction
            $productEntry = ProductEntry::where('product_id', $transaction->product_id)
                ->where('created_at', $transaction->created_at)
                ->first();

            if ($productEntry) {
                Log::info("rah lqit entry");
                $expiryDate = $productEntry->expiry_date;
            }
        }
        return view('transactions.edit', compact('transaction', 'products', 'expiryDate'));
    }
    // public function update(Request $request, $id)
    // {
    //     // dd($request);
    //     // $user = Auth::user();
    //     Log::info("ANA HNA", []);

    //     // Find the existing transaction by ID
    //     $transaction = Transaction::find($id);

    //     // Check if the transaction exists
    //     if (!$transaction) {
    //         return redirect()->route('transactions.index')->withErrors(['transaction' => 'The selected transaction does not exist.']);
    //     }

    //     // Assume the field for product name in the form is 'product_name'
    //     $productName = $request->input('product_name');

    //     // Find the product by name
    //     $product = Product::where('name', $productName)->first();
    //     Log::info("ANA HNA flkher`$productName`", []);

    //     // Check if product exists
    //     if (!$product) {
    //         return back()->withErrors(['product_name' => 'The selected product does not exist.'])->withInput();
    //     }

    //     // Validate other request data
    //     $validatedData = $request->validate([
    //         'quantity' => 'required|integer|min:1',
    //         'type' => 'required|in:entry,exit',
    //         'remarks' => 'nullable|string',
    //         'expiry_date' => 'required_if:type,entry|date|nullable'
    //     ]);
    //     // DB::beginTransaction();

    //     try {
    //         // if ($transaction->type === 'entry') {
    //         //     // Handle updating entry transaction
    //         //     $difference = $validatedData['quantity'] - $transaction->quantity;
    //         //     $entry = ProductEntry::where('product_id', $product->id)
    //         //         ->where('expiry_date', $transaction->expiry_date)
    //         //         ->first();

    //         //     if ($entry) {
    //         //         $entry->update(['quantity' => $entry->quantity + $difference]);
    //         //     }
    //         // } else {
    //         //     // Handle updating exit transaction
    //         //     $difference = $transaction->quantity - $validatedData['quantity'];
    //         //     $productEntries = ProductEntry::where('product_id', $product->id)
    //         //         ->orderBy('expiry_date', 'asc')
    //         //         ->get();

    //         //     if ($difference > 0) {
    //         //         $remainingQuantity = $difference;
    //         //         foreach ($productEntries as $entry) {
    //         //             if ($remainingQuantity <= $entry->quantity) {
    //         //                 $entry->increment('quantity', $remainingQuantity);
    //         //                 $remainingQuantity = 0;
    //         //                 break;
    //         //             } else {
    //         //                 $remainingQuantity -= $entry->quantity;
    //         //                 $entry->update(['quantity' => 0]);
    //         //             }
    //         //         }
    //         //     } elseif ($difference < 0) {
    //         //         $remainingQuantity = abs($difference);
    //         //         foreach ($productEntries as $entry) {
    //         //             if ($remainingQuantity <= $entry->quantity) {
    //         //                 $entry->decrement('quantity', $remainingQuantity);
    //         //                 $remainingQuantity = 0;
    //         //                 break;
    //         //             } else {
    //         //                 $remainingQuantity -= $entry->quantity;
    //         //                 $entry->update(['quantity' => 0]);
    //         //             }
    //         //         }
    //         //     }
    //         // }
    //         Log::info("ANA HNA flkher", []);

    //         // Find the corresponding ProductEntry
    //         $productEntry = ProductEntry::where('product_id', $transaction->product_id)
    //             ->where('created_at', $transaction->created_at)
    //             ->first();

    //         if (!$productEntry) {
    //             throw new \Exception("No corresponding product entry found for this transaction.");
    //         }

    //         // Update the expiry date
    //         $productEntry->update([
    //             'expiry_date' => $validatedData['expiry_date'],
    //         ]);

    //         // Update transaction properties
    //         // $transaction->quantity = $validatedData['quantity'];
    //         $transaction->remarks = $validatedData['remarks'] ?? $transaction->remarks;
    //         $transaction->save();

    //         // DB::commit();

    //         // Redirect to the transactions list with a success message
    //         return redirect()->route('transactions.index')->with([
    //             'message' => 'Transaction successfully updated.',
    //             'alert-type' => 'success'
    //         ]);
    //     } catch (\Exception $e) {
    //         DB::rollBack();
    //         Log::error('Transaction update failed', ['transactionID' => $transaction->id, 'error' => $e->getMessage()]);
    //         return back()->withErrors(['error' => 'Failed to update transaction. Please try again.']);
    //     }
    // }

    public function update(Request $request, $id)
    {
        Log::info("Transaction update started", ['transaction_id' => $id]);

        $transaction = Transaction::findOrFail($id);

        $validatedData = $request->validate([
            'expiry_date' => 'required_if:type,entry|date|nullable',
            'remarks' => 'nullable|string',
        ]);

        DB::beginTransaction();

        try {
            if ($transaction->type === 'entry') {
                $productEntry = ProductEntry::where('product_id', $transaction->product_id)
                    ->where('quantity', $transaction->quantity)
                    ->where('created_at', $transaction->created_at)
                    ->first();

                if (!$productEntry) {
                    throw new \Exception("No corresponding product entry found for this transaction.");
                }

                $productEntry->update([
                    'expiry_date' => $validatedData['expiry_date'],
                ]);

                Log::info("ProductEntry updated", ['product_entry_id' => $productEntry->id]);
            }

            $transaction->update([
                'remarks' => $validatedData['remarks'] ?? $transaction->remarks,
            ]);

            Log::info("Transaction updated", ['transaction_id' => $transaction->id]);

            DB::commit();

            return redirect()->route('transactions.index')->with([
                'message' => 'Transaction successfully updated.',
                'alert-type' => 'success'
            ]);
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Transaction update failed', ['transaction_id' => $transaction->id, 'error' => $e->getMessage()]);
            return back()->withErrors(['error' => 'Failed to update transaction. ' . $e->getMessage()]);
        }
    }
    public function destroy($id)
    {
        $transaction = Transaction::find($id);

        if ($transaction) {
            DB::beginTransaction();

            try {
                if ($transaction->type === 'entry') {
                    // Define a margin for created_at comparison
                    $marginInSeconds = 5;
                    $transactionCreatedAt = $transaction->created_at;
                    $startTime = $transactionCreatedAt->copy()->subSeconds($marginInSeconds);
                    $endTime = $transactionCreatedAt->copy()->addSeconds($marginInSeconds);

                    // Handle the case of an entry transaction
                    $productEntry = ProductEntry::where('product_id', $transaction->product_id)
                        ->where('created_at', '>=', $startTime)
                        ->where('created_at', '<=', $endTime)
                        ->first();
                    Log::info('Product Entry', ['product_entry' => $productEntry]);
                    if ($productEntry) {
                        // Check if the entry has been used in another exit transaction
                        $usedInTransactions = DB::table('transaction_product_entry')
                            ->where('product_entry_id', $productEntry->id)
                            ->count();
                        Log::info('Used in transactions', ['usedInTransactions' => $usedInTransactions]);

                        if ($usedInTransactions > 0) {
                            throw new \Exception("This product entry has been used in an exit transaction and cannot be deleted.");
                        }

                        $productEntry->delete();
                        Log::info('Entry transaction product entry deleted', ['product_entry_id' => $productEntry->id]);
                    }
                } else {
                    // Handle the case of an exit transaction
                    foreach ($transaction->productEntries as $entry) {
                        $entry->quantity += $entry->pivot->quantity;
                        $entry->save();
                    }
                }

                $transaction->delete();

                DB::commit();

                return redirect()->route('transactions.index')->with('message', 'Transaction deleted successfully');
            } catch (\Exception $e) {
                DB::rollBack();
                Log::error('Transaction delete failed', ['transaction_id' => $transaction->id, 'error' => $e->getMessage()]);
                return back()->with([
                    'message' => 'Failed to process transaction: ' . $e->getMessage(),
                    'alert-type' => 'error'
                ]);
            }
        } else {
            return redirect()->route('transactions.index')->with('message', 'Transaction not found');
        }
    }
}
